########################################################################################################################################################################################################
###Autor: Miguel Zatarain
###Fecha: 04/05/2015
###Aceleracin en el eje x
###Script que va a calcular la aceleracin en el eje x durante una terapia en funcion del tiempo
############################################################################################################################
#Obtenemos los valores de la tabla para seleccionar la terapia la identidad del paciente y el tipo de terapia que queremos
args <- commandArgs(TRUE)
require(RODBC)
require(FKF) #Para filtrar la aceleracin
conn <- odbcDriverConnect("driver=SQL Server;database=BeataMaria;server=ZARU\\SQLEXPRESS;uid=sistac;pwd=sistac")

#Valores de tiempo y numero de terapia
tminx<-as.numeric(args[2])
tmaxx<-as.numeric(args[3])
n<-as.numeric(args[4])

#Accedemos a la tabla para obtener los valores
b<-paste("SELECT     IdMuestra, PosicionX, IdTerapia, VelocidadX, Tiempo, PuntoDeseadoX
FROM         Muestra
WHERE     (IdTerapia = ",n,")
ORDER BY IdMuestra",sep="")

Data.fr <- sqlQuery(conn, b, errors=T)
attach(Data.fr)
a<-max(dim(Data.fr))   #miramos la longitud de a
 
#Calculamos el tiempo
ti<-as.character(Tiempo)
s1 <- as.numeric(sapply(strsplit(ti, split=':', fixed=TRUE), function(x) (x[1])))
s2 <- as.numeric(sapply(strsplit(ti, split=':', fixed=TRUE), function(x) (x[2])))
s3 <- as.numeric(sapply(strsplit(ti, split=':', fixed=TRUE), function(x) (x[3])))
Vtiempois<-s1*3600+s2*60+s3
#Ya tenemos el tiempo en segundos
#Vamos a quitar los primeros 5 valores de cada columna porque tienen un error que puede llevar a errores
#Asignamos los valores de cada columna a una variable
Vtiempo<-Vtiempois[4:a]

#pasamos las variables de la tabla a variables internas

vx<-VelocidadX
vx<-(vx[5:a])/1000
px<-PosicionX
px<-(px[5:a])/1000

pdx<-PuntoDeseadoX
pdx<-(pdx[5:a])

maxleng<-length(pdx)
#Calibramos la posicion del robot
pxo<-(rep(1,a-4))
pxf<-(pdx+pxo)/1000

aceleracionx<-diff(vx)/(diff(Vtiempo))
aceleracionx[is.na(aceleracionx)] <- 0
aceleracionx[is.infinite(aceleracionx)] <- 1

#hacemos un filtro para coger solo los valotes en el tiempo que tenemos
filtro<-(maxleng:1)*0

for(i in 1:maxleng){
	if((Vtiempo[i]< tmaxx) & (Vtiempo[i]>tminx)){
		filtro[i]<-i
	}
}

#A veces en los valores iniciales da un error de na, asique lo correguimos

aceleracionx<-aceleracionx[filtro]
tiempo<-Vtiempo[filtro]
pxf<-pxf[filtro]
px<-px[filtro]

y <- aceleracionx
y[c(3, 10)] <- NA  # Para que los valores NA sean tratados
## Inicializamos las constantes en los parmetros:
dt <- ct <- matrix(0)
Zt <- Tt <- matrix(1)
a0 <- y[1]            # Variacin del primer valor
P0 <- matrix(100)     # variacin de a0
## Parametros estimados:
fit.fkf <- optim(c(HHt = var(y, na.rm = TRUE) * .5,
GGt = var(y, na.rm = TRUE) * .5),
fn = function(par, ...)
-fkf(HHt = matrix(par[1]), GGt = matrix(par[2]), ...)$logLik,
yt = rbind(y), a0 = a0, P0 = P0, dt = dt, ct = ct,
Zt = Zt, Tt = Tt, check.input = FALSE)
## Filtramos la aceleracion con los parametros estimados:
amaxxfkf<-fkf(a0, P0, dt, ct, Tt, Zt, HHt = matrix(fit.fkf$par[1]),
GGt = matrix(fit.fkf$par[2]), yt = rbind(y))
plotax<-amaxxfkf$att[1, ]


#graficamos


if(as.numeric(args[5])==0){
png(paste(args[1],"R\\AceleracionxTerapia",n,".png",sep=""),width = 1147, height = 756, units = "px", pointsize = 16)

}
if(as.numeric(args[5])==1){
png(paste(args[1],"R\\Muestras1.png",sep=""),width = 1700, height = 600, units = "px", pointsize = 16)

}
if(as.numeric(args[5])==2){
png(paste(args[1],"R\\Muestras2.png",sep=""),width = 1700, height = 600, units = "px", pointsize = 16)

}

maxy<-max(max(px),max(pxf))
miny<-min(min(px),min(pxf))
par(mfrow=c(2,1))
x <- c(min(tiempo),max(tiempo)); y <- c(miny,1.4*maxy);
plot(x, y,type="b", pch=21, col="white", lty=3, xlab="Tiempo(s)", ylab="Desplazamientos en x(m)")
lines((tiempo),pxf[1:length(tiempo)],type="overplotted", pch=2, col="red")
lines((tiempo),px[1:length(tiempo)],type="overplotted", pch=1, col="green")
main<-"Desplazamiento del paciente en el eje x (m)"
sub<-paste("")
title(main, sub)
legend("topleft",legend=c("Puntos realizados por el paciente en el eje x (m)","Puntos deseados del paciente en el eje x (m)"),box.lty=1,pch=c(1,2),col=c("green","red"))


maxy<-max(plotax)
miny<-min(plotax)
x <- c(min(tiempo),max(tiempo)); y <- c(miny,1.4*maxy);
maxy<-1.4*maxy 
plot(x, y,type="b", pch=21, col="white", lty=3, xlab="Tiempo(s)", ylab="Aceleracin (m/s^2)")
lines((tiempo),plotax[1:length(tiempo)], type="overplotted",pch=1, col="blue",xlab="Tiempo(s)", ylab="Aceleraciin en el eje x (m^2/s)")
main<-"Aceleracin del paciente en el eje x"
sub<-paste(" NTerapia: ",n,sep="")
title(main, sub)
legend("topleft",legend=c("Aceleracin del paciente en el eje x (m^2/s)"),box.lty=1,pch=c(1),col=c("blue"))
dev.off()


rm(sub,plotax,a0,conn,amaxxfkf,ct,dt,fit.fkf,P0,Tt,Zt,aceleracionx,px,maxy,miny,main,x,y,pdx,vx,tiempo,i,maxleng,filtro,Vtiempo,tmaxx,tminx,pxf,pxo,Vtiempois,ti,s1,s2,s3,Data.fr,a,b,n)

