########################################################################################################################################################################################################
###Autor: Miguel Zatarain
###Fecha: 04/05/2015
###Aceleracin en el eje y
###Script que va a calcular la aceleracin en el eje y durante una terapia en funcion del tiempo
############################################################################################################################
#Obtenemos los valores de la tabla para seleccionar la terapia la identidad del paciente y el tipo de terapia que queremos
args <- commandArgs(TRUE)
require(RODBC)
require(FKF) #Para filtrar la aceleracin
conn <- odbcDriverConnect("driver=SQL Server;database=BeataMaria;server=ZARU\\SQLEXPRESS;uid=sistac;pwd=sistac")

#Valores de tiempo y numero de terapia

tminy<-as.numeric(args[2])
tmaxy<-as.numeric(args[3])
n<-as.numeric(args[4])

#Accedemos a la tabla para obtener los valores
b<-paste("SELECT     IdMuestra, PosicionY, IdTerapia, VelocidadY, Tiempo, PuntoDeseadoY
FROM         Muestra
WHERE     (IdTerapia = ",n,")
ORDER BY IdMuestra",sep="")

Data.fr <- sqlQuery(conn, b, errors=T)
attach(Data.fr)
a<-max(dim(Data.fr))   #miramos la longitud de a
 
#Calculamos el tiempo
ti<-as.character(Tiempo)
s1 <- as.numeric(sapply(strsplit(ti, split=':', fixed=TRUE), function(x) (x[1])))
s2 <- as.numeric(sapply(strsplit(ti, split=':', fixed=TRUE), function(x) (x[2])))
s3 <- as.numeric(sapply(strsplit(ti, split=':', fixed=TRUE), function(x) (x[3])))
Vtiempois<-s1*3600+s2*60+s3
#Ya tenemos el tiempo en segundos
#Vamos a quitar los primeros 5 valores de cada columna porque tienen un error que puede llevar a errores
#Asignamos los valores de cada columna a una variable
Vtiempo<-Vtiempois[4:a]

#pasamos las variables de la tabla a variables internas

vy<-VelocidadY
vy<-(vy[5:a])/1000
py<-PosicionY
py<-(py[5:a])/1000

pdy<-PuntoDeseadoY
pdy<-(pdy[5:a])

maxleng<-length(pdy)
#Calibramos la posicion del robot
pyo<-rep(600,a-4)
pyf<-(pyo-pdy)/1000

aceleraciony<-diff(vy)/(diff(Vtiempo))
aceleraciony[is.na(aceleraciony)] <- 0
aceleraciony[is.infinite(aceleraciony)] <- 1

#hacemos un filtro para coger solo los valotes en el tiempo que tenemos
filtro<-(maxleng:1)*0

for(i in 1:maxleng){
	if((Vtiempo[i]< tmaxy) & (Vtiempo[i]>tminy)){
		filtro[i]<-i
	}
}

#A veces en los valores iniciales da un error de na, asique lo correguimos

aceleraciony<-aceleraciony[filtro]
tiempo<-Vtiempo[filtro]
pyf<-pyf[filtro]
py<-py[filtro]

y <- aceleraciony
y[c(3, 10)] <- NA  # Para que los valores NA sean tratados
## Inicializamos las constantes en los parmetros:
dt <- ct <- matrix(0)
Zt <- Tt <- matrix(1)
a0 <- y[1]            # Variacin del primer valor
P0 <- matrix(100)     # variacin de a0
## Parametros estimados:
fit.fkf <- optim(c(HHt = var(y, na.rm = TRUE) * .5,
GGt = var(y, na.rm = TRUE) * .5),
fn = function(par, ...)
-fkf(HHt = matrix(par[1]), GGt = matrix(par[2]), ...)$logLik,
yt = rbind(y), a0 = a0, P0 = P0, dt = dt, ct = ct,
Zt = Zt, Tt = Tt, check.input = FALSE)
## Filtramos la aceleracion con los parametros estimados:
amaxyfkf<-fkf(a0, P0, dt, ct, Tt, Zt, HHt = matrix(fit.fkf$par[1]),
GGt = matrix(fit.fkf$par[2]), yt = rbind(y))
plotay<-amaxyfkf$att[1, ]


#graficamos

if(as.numeric(args[5])==0){
png(paste(args[1],"R\\AceleracionyTerapia",n,".png",sep=""),width = 1147, height = 756, units = "px", pointsize = 16)

}
if(as.numeric(args[5])==1){
png(paste(args[1],"R\\Muestras1.png",sep=""),width = 1700, height = 600, units = "px", pointsize = 16)

}
if(as.numeric(args[5])==2){
png(paste(args[1],"R\\Muestras2.png",sep=""),width = 1700, height = 600, units = "px", pointsize = 16)

}

maxy<-max(max(py),max(pyf))
miny<-min(min(py),min(pyf))
par(mfrow=c(2,1))
x <- c(min(tiempo),max(tiempo)); y <- c(miny,1.4*maxy);
plot(x, y,type="b", pch=21, col="white", lty=3, xlab="Tiempo(s)", ylab="Desplazamientos en y(m)")
lines((tiempo),pyf[1:length(tiempo)],type="overplotted", pch=2, col="red")
lines((tiempo),py[1:length(tiempo)],type="overplotted", pch=1, col="green")
main<-"Desplazamiento del paciente en el eje y (m)"
sub<-paste("")
title(main, sub)
legend("topleft",legend=c("Puntos realizados por el paciente en el eje y (m)","Puntos deseados del paciente en el eje y (m)"),box.lty=1,pch=c(1,2),col=c("green","red"))


maxy<-max(plotay)
miny<-min(plotay)
x <- c(min(tiempo),max(tiempo)); y <- c(miny,1.4*maxy);
maxy<-1.4*maxy 
plot(x, y,type="b", pch=21, col="white", lty=3, xlab="Tiempo(s)", ylab="Aceleracin (m/s^2)")
lines((tiempo),plotay[1:length(tiempo)], type="overplotted",pch=1, col="blue",xlab="Tiempo(s)", ylab="Aceleracin en el eje y (m^2/s)")
main<-"Aceleracin del paciente en el eje y"
sub<-paste(" NTerapia: ",n,sep="")
title(main, sub)
legend("topleft",legend=c("Aceleraciin del paciente en el eje y (m^2/s)"),box.lty=1,pch=c(1),col=c("blue"))
dev.off()


rm(a0,conn,amaxyfkf,ct,px,sub,dt,fit.fkf,P0,plotay,tmaxy,tminy,Tt,Zt,aceleraciony,py,maxy,miny,main,x,y,pdy,vy,tiempo,i,maxleng,filtro,Vtiempo,pyf,pyo,Vtiempois,ti,s1,s2,s3,Data.fr,a,b,n)

