########################################################################################################################################################################################################
###Autor: Miguel Zatarain
###Fecha: 28/04/2015
###Fuerza en el eje x
###Script que va a calcular la fuerza que realiza el paciente y con la que asiste el robot en el eje x durante una terapia
############################################################################################################################
args <- commandArgs(TRUE)
require(RODBC)
conn <- odbcDriverConnect("driver=SQL Server;database=BeataMaria;server=ZARU\\SQLEXPRESS;uid=sistac;pwd=sistac")

require(FKF) #Para filtrar la fuerza vamos a utilizar el filitro de kalman


fmedx<-as.numeric(args[4])
fmaxx<-as.numeric(args[2])
fminx<-as.numeric(args[3])
n<-as.numeric(args[5])

b<-paste("SELECT     IdMuestra, PosicionX, FuerzaX, IdTerapia, FuerzaAsistiva, VelocidadX, Tiempo, PuntoDeseadoX, PuntoDeseadoY
FROM         Muestra
WHERE     (IdTerapia = ",n,")
ORDER BY IdMuestra",sep="")

Data.fr <- sqlQuery(conn, b, errors=T)
attach(Data.fr)
a<-max(dim(Data.fr))
m<-1								# En Kg
b<-18								# En Kg/s

#debido a que el tiempo es un caracter vamos a transformarlo a numrico en segundos mediante un bucle
ti<-as.character(Tiempo)
s1 <- as.numeric(sapply(strsplit(ti, split=':', fixed=TRUE), function(x) (x[1])))
s2 <- as.numeric(sapply(strsplit(ti, split=':', fixed=TRUE), function(x) (x[2])))
s3 <- as.numeric(sapply(strsplit(ti, split=':', fixed=TRUE), function(x) (x[3])))
Vtiempois<-s1*3600+s2*60+s3
#Ya tenemos el tiempo en segundos
#Vamos a quitar los primeros 5 valores de cada columna porque tienen un error que puede llevar a errores
#Asignamos los valores de cada columna a una variable
Vtiempo<-Vtiempois[4:a]

Fx<-FuerzaX
Fx<-(Fx[5:a])/1000


k<-FuerzaAsistiva 		# En Kg/s^2
k<-k[5:a]

vx<-VelocidadX
vx<-(vx[4:a])/1000


px<-PosicionX
px<-px[4:a]

pdx<-PuntoDeseadoX
pdx<-(pdx[5:a])
pdy<-PuntoDeseadoY
pdy<-(pdy[5:a])

#Calibramos la posicion del robot
pxo<-(rep(1,a-4))
pxf<-(pdx+pxo)/1000

maxleng<-length(pdx)

#Calibramos la posicion del robot
pxo<-rep(1,a-3)
pxf<-(pdx+pxo)/1000

maxleng<-(length(Fx))
aceleracionx<-diff(vx)/(diff(Vtiempo))
aceleracionx[is.na(aceleracionx)] <- 0
aceleracionx[is.infinite(aceleracionx)] <- 1
Fdx<- m*aceleracionx + b*vx + k*((pxf - px))	

#Dado que el tiempo de muestro es distinto cada vez no nos da los mismo puntos exactos, asique para saber donde
#empieza una vuelta seleccionamos un pequeno area
Iniciox<-pdx[150]
Inicioy<-pdy[150]
filtro<-(1:maxleng)*0
for(i in 1:(maxleng)){ 
	if(Iniciox>(pdx[i]-30) & Inicioy>(pdy[i]-30) & Iniciox<(pdx[i]+30) & Inicioy<pdy[i]+30) {
	filtro[i]=i}
}

#Filtramos para que solo se produzcan varios de los puntos de ese area seguidos

filtrob<-filtro*0

for(i in 1:(maxleng)){
	if (i>110){
	if((filtro[i]!=0)&(filtro[i-1]!=0)&(filtro[i-2]!=0)&(filtro[i-3]!=0)&(filtro[i-4]!=0)&(filtro[i-5]!=0)&(filtro[i-6]!=0)){
	filtrob[i]=i}
	}
}

#Calculamos los lmites de inicio de cada vuelta
a=150
b<-1
limites<-vector()
for(i in 1:(maxleng)){ 
	if(filtrob[i]>a+650) {
		limites[b]=i
		b<-b+1
		a<-i}
}

#Inicializamos los vectores para calcular los puntos mximos mnimo y medio


Fuerzasx1<-vector()
Fuerzasx2<-vector()
mediafuexcompara<-mean(abs(Fx))
mediafuex<-Fx[limites[1]:limites[2]]
maxfuex<-Fx[limites[1]:limites[2]]
minfuex<-Fx[limites[1]:limites[2]]
minpdxfue<-pxf[limites[1]:limites[2]]
maxpdxfue<-pxf[limites[1]:limites[2]]
mediapdxfue<-pxf[limites[1]:limites[2]]
Vtiempofx1<-Vtiempo[limites[1]:limites[2]]
Vtiempofx2<-Vtiempo[limites[1]:limites[2]]
Vtiempofx3<-Vtiempo[limites[1]:limites[2]]

Fdsx1<-vector()
Fdsx2<-vector()
mediafuedx<-Fdx[limites[1]:limites[2]]
maxfuedx<-Fdx[limites[1]:limites[2]]
minfuedx<-Fdx[limites[1]:limites[2]]



for(i in 1:(length(limites))){
	if(i<length(limites)-1){

			#Calculamos las velocidades
		
			Fuerzasx1<-Fx[limites[i]:limites[i+1]]
			Fuerzasx2<-Fx[limites[i+1]:limites[i+2]]
			Fdsx1<-Fdx[limites[i]:limites[i+1]]
			Fdsx2<-Fdx[limites[i+1]:limites[i+2]]			
			

			if(mean(abs(Fuerzasx1)) < mean(abs(Fuerzasx2)) & mean(abs(Fuerzasx2))>mean(abs(maxfuex))){
			maxfuex<-Fuerzasx2
			maxfuedx<-Fdsx2
			maxpdxfue<-pxf[limites[i+1]:limites[i+2]]
			Vtiempofx1<-Vtiempo[limites[i+1]:limites[i+2]]
			}
			
			if(mean(abs(Fuerzasx1))> mean(abs(Fuerzasx2))& mean(abs(Fuerzasx2))<mean(abs(minfuex))){
			minfuex<-Fuerzasx2
			minfuedx<-Fdsx2
			minpdxfue<-pxf[limites[i+1]:limites[i+2]]
			Vtiempofx2<-Vtiempo[limites[i+1]:limites[i+2]]
			}

			if(abs(1-(abs(mean(abs(Fuerzasx1))/mediafuexcompara))) > abs(1-(abs(mean(abs(Fuerzasx2))/mediafuexcompara)))& abs(1-(abs(mean(abs(Fuerzasx2))/mediafuexcompara)))< abs(1-(abs(mean(abs(mediafuex))/mediafuexcompara)))){
			mediafuex<-Fuerzasx2
			mediafuedx<-Fdsx2
			mediapdxfue<-pxf[limites[i+1]:limites[i+2]]
			Vtiempofx3<-Vtiempo[limites[i+1]:limites[i+2]]
			}			
}
}

mediapdxfue<-mediapdxfue*10
minpdxfue<-minpdxfue*10
maxpdxfue<-maxpdxfue*10
Vtiempofx1<-Vtiempofx1 - Vtiempofx1[1]
Vtiempofx2<-Vtiempofx2 - Vtiempofx2[1]
Vtiempofx3<-Vtiempofx3 - Vtiempofx3[1]

maxtiempo<-max(max(Vtiempofx1),max(Vtiempofx2),max(Vtiempofx3))

Vtiempofx1<-Vtiempofx1*maxtiempo/max(Vtiempofx1)
Vtiempofx2<-Vtiempofx2*maxtiempo/max(Vtiempofx2)
Vtiempofx3<-Vtiempofx3*maxtiempo/max(Vtiempofx3)

#Empezamos a graficar
#Calculamos el mayor valor de de x para el ploteo
maxy<-max(max(maxfuex),max(minfuex),max(mediafuex),6+max(mediapdxfue),max(maxfuedx),max(minfuedx),max(mediafuedx))
miny<-min(min(maxfuex),min(minfuex),min(mediafuex),min(mediapdxfue),min(maxfuedx),min(minfuedx),min(mediafuedx))



if(as.numeric(args[6])==0){
png(paste(args[1],"R\\FuerzasvxTerapia",n,".png",sep=""),width = 1147, height = 756, units = "px", pointsize = 16)

}
if(as.numeric(args[6])==1){
png(paste(args[1],"R\\Muestras1.png",sep=""),width = 1275, height = 450, units = "px", pointsize = 16)

}
if(as.numeric(args[6])==2){
png(paste(args[1],"R\\Muestras2.png",sep=""),width = 1275, height = 450, units = "px", pointsize = 16)

}

par(mar=c(5, 4, 4, 5) + 0.1)

x <- c(1,max(Vtiempofx1)); y <- c(miny,1.4*maxy);
maxy<-1.4*maxy 
plot(x, y,type="b", pch=21, col="white",xaxt="n", lty=3, xlab="Tiempo(s)", ylab="Fuerzas (N)")
axis(1,at=c(1:max(Vtiempofx1 )))
mtext("Distancia (dm)", side=4, line=3, cex.lab=1,las=0)

if(fmaxx>0.5){
####################################################
#Filtro para la fuerza

y <- maxfuex
y[c(3, 10)] <- NA  # Para que los valores NA sean tratados
## Inicializamos las constantes en los parmetros:
dt <- ct <- matrix(0)
Zt <- Tt <- matrix(1)
a0 <- y[1]            # Variacin del primer valor
P0 <- matrix(10)     # variacin de a0
## Parametros estimados:
fit.fkf <- optim(c(HHt = var(y, na.rm = TRUE) * .5,
GGt = var(y, na.rm = TRUE) * .5),
fn = function(par, ...)
-fkf(HHt = matrix(par[1]), GGt = matrix(par[2]), ...)$logLik,
yt = rbind(y), a0 = a0, P0 = P0, dt = dt, ct = ct,
Zt = Zt, Tt = Tt, check.input = FALSE)
## Filtramos la aceleracion con los parametros estimados:
Fxf<-fkf(a0, P0, dt, ct, Tt, Zt, HHt = matrix(fit.fkf$par[1]),
GGt = matrix(fit.fkf$par[2]), yt = rbind(y))
############################################################
maxfuexf<-Fxf$att[1, ]

axis(4, at= 6 + seq((miny-6),(maxy-6), by=2),labels=round(seq(miny-6, maxy-6, by=2),digits=2), las=2, tck=-.01)

lines((Vtiempofx1),maxfuexf[1:length(Vtiempofx1)], type="overplotted",pch=1, col="deepskyblue")
lines((Vtiempofx1),maxfuedx[1:length(Vtiempofx1)], type="overplotted",pch=2, col="dodgerblue4")
lines((Vtiempofx1),6+maxpdxfue[1:length(Vtiempofx1)],type="overplotted", pch=4, col="black")

legend("topleft",legend=c("Fuerza mxima del paciente en el eje x(N)","Fuerza asistiva en el eje x(N)","Puntos deseados del paciente en el eje x(dm)"),box.lty=1,pch=c(1,2,4),col=c("deepskyblue","dodgerblue4","black"))


}
if(fminx>0.5){

####################################################
#Filtro para la fuerza

y <- minfuex
y[c(3, 10)] <- NA  # Para que los valores NA sean tratados
## Inicializamos las constantes en los parmetros:
dt <- ct <- matrix(0)
Zt <- Tt <- matrix(1)
a0 <- y[1]            # Variacin del primer valor
P0 <- matrix(10)     # variacin de a0
## Parametros estimados:
fit.fkf <- optim(c(HHt = var(y, na.rm = TRUE) * .5,
GGt = var(y, na.rm = TRUE) * .5),
fn = function(par, ...)
-fkf(HHt = matrix(par[1]), GGt = matrix(par[2]), ...)$logLik,
yt = rbind(y), a0 = a0, P0 = P0, dt = dt, ct = ct,
Zt = Zt, Tt = Tt, check.input = FALSE)
## Filtramos la aceleracion con los parametros estimados:
Fminxf<-fkf(a0, P0, dt, ct, Tt, Zt, HHt = matrix(fit.fkf$par[1]),
GGt = matrix(fit.fkf$par[2]), yt = rbind(y))
############################################################
minfuexf<-Fminxf$att[1, ]
axis(4, at= 6 + seq((miny-6),(maxy-6), by=2),labels=round(seq(miny-6, maxy-6, by=2),digits=2), las=2, tck=-.01)

lines((Vtiempofx2),minfuexf[1:length(Vtiempofx2)],type="overplotted", pch=3, col="firebrick1")
lines((Vtiempofx2),minfuedx[1:length(Vtiempofx2)],type="overplotted", pch=5, col="firebrick4")
lines((Vtiempofx2),6+minpdxfue[1:length(Vtiempofx2)],type="overplotted", pch=4, col="black")

legend("topleft",legend=c("Fuerza mnima del paciente en el eje x(N)","Fuerza asistiva en el eje x(N)","Puntos deseados del paciente en el eje x(dm)"),box.lty=1,pch=c(3,5,4),col=c("firebrick1","firebrick4","black"))

}
if(fmedx>0.5){

####################################################
#Filtro para la fuerza

y <- mediafuex
y[c(3, 10)] <- NA  # Para que los valores NA sean tratados
## Inicializamos las constantes en los parmetros:
dt <- ct <- matrix(0)
Zt <- Tt <- matrix(1)
a0 <- y[1]            # Variacin del primer valor
P0 <- matrix(10)     # variacin de a0
## Parametros estimados:
fit.fkf <- optim(c(HHt = var(y, na.rm = TRUE) * .5,
GGt = var(y, na.rm = TRUE) * .5),
fn = function(par, ...)
-fkf(HHt = matrix(par[1]), GGt = matrix(par[2]), ...)$logLik,
yt = rbind(y), a0 = a0, P0 = P0, dt = dt, ct = ct,
Zt = Zt, Tt = Tt, check.input = FALSE)
## Filtramos la aceleracion con los parametros estimados:
Fmediaxf<-fkf(a0, P0, dt, ct, Tt, Zt, HHt = matrix(fit.fkf$par[1]),
GGt = matrix(fit.fkf$par[2]), yt = rbind(y))
############################################################
mediafuexf<-Fmediaxf$att[1, ]
axis(4, at= 6 + seq((miny-6),(maxy-6), by=2),labels=round(seq(miny-6, maxy-6, by=2),digits=2), las=2, tck=-.01)

lines((Vtiempofx3),mediafuexf[1:length(Vtiempofx3)],type="overplotted", pch=6, col="darkolivegreen1")
lines((Vtiempofx3),mediafuedx[1:length(Vtiempofx3)],type="overplotted", pch=7, col="darkgreen")
lines((Vtiempofx3),6+mediapdxfue[1:length(Vtiempofx3)],type="overplotted", pch=4, col="black")

legend("topleft",legend=c("Fuerza media del paciente en el eje x(N)","Fuerza asistiva en el eje x(N)","Puntos deseados del paciente en el eje x(dm)"),box.lty=1,pch=c(6,7,4),col=c("darkolivegreen1","darkgreen","black"))

}

main<-"Fuerza del paciente en el eje x"
sub<-paste(" NTerapia: ",n,sep="")
title(main, sub)
dev.off()

rm(n,fmedx,limites,filtrob,fmaxx,fminx,Data.fr,a,m,b,ti,s1,s2,s3,Vtiempois,Vtiempo,Fx,k,vx,vxo,vxf,pdx,pdy,maxleng,aceleracionx,Fdx,Iniciox,Inicioy,filtro,i,Fuerzasx1,Fuerzasx2,mediafuexcompara,mediafuex,maxfuex,minfuex,minpdxfue,maxpdxfue,mediapdxfue,Vtiempofx1,Vtiempofx2,Vtiempofx3,Fdsx1,Fdsx2,mediafuedx,maxfuedx,minfuedx,maxtiempo,maxy,miny,x,y,dt,ct,Zt,Tt,a0,P0,Fxf,maxfuexf,Fminxf,minfuexf,Fmediaxf,mediafuexf,main,sub,px,pxf,pxo)
rm(fit.fkf)